//This file is part of Glest Shared Library (www.glest.org)
//Copyright (C) 2005 Matthias Braun <matze@braunis.de>

//You can redistribute this code and/or modify it under 
//the terms of the GNU General Public License as published by the Free Software 
//Foundation; either version 2 of the License, or (at your option) any later 
//version.

#include "socket.h"

#include <stdexcept>
#include <sstream>

#include "conversion.h"

using namespace std;
using namespace Shared::Util;

namespace Shared{ namespace Platform{

// ===============================================
//	class Socket
// ===============================================

Socket::Socket(int sock){
	this->sock= sock;
}

Socket::Socket(){
	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(sock < 0) {
		throwException("Error creating socket");
	}
}

Socket::~Socket() {
	close();
}

void Socket::close() {
	::close(sock);
}

void Socket::send(const void *data, int dataSize) {
	int bytesSent= ::send(sock, reinterpret_cast<const char*>(data), dataSize, 0);
	if(bytesSent!=dataSize){
		throwException("Error sending socket data");
	}
}

void Socket::receive(void *data, int dataSize) {
	int bytesReceived= recv(sock, reinterpret_cast<char*>(data), dataSize, 0);
	if(bytesReceived!=dataSize){
		throwException("Error receiving socket data");
	}
}

string Socket::getHostname() const {
	const int strSize= 256;
	char hostname[strSize];
	gethostname(hostname, strSize);
	return hostname;
}

void Socket::throwException(const string &str){
	std::stringstream msg;
	msg << str << " (Error: " << strerror(errno) << ")";
	throw runtime_error(msg.str());
}

// ===============================================
//	class ClientSocket
// ===============================================

void ClientSocket::connect(const string &host, int port){
	sockaddr_in addr;
	memset(&addr, 0, sizeof(addr));

    addr.sin_family= AF_INET;
	addr.sin_addr.s_addr= inet_addr(host.c_str());
	addr.sin_port= htons(port);

	int err= ::connect(sock, reinterpret_cast<const sockaddr*>(&addr), sizeof(addr));
	if(err < 0) {
		throwException("Error connecting socket");
	}
}

// ===============================================
//	class ServerSocket
// ===============================================

void ServerSocket::bind(int port){
	//sockaddr structure
	sockaddr_in addr;
	addr.sin_family= AF_INET;
	addr.sin_addr.s_addr= INADDR_ANY;
	addr.sin_port= htons(port);
	
	int err= ::bind(sock, reinterpret_cast<sockaddr*>(&addr), sizeof(addr));
	if(err < 0) {
		throwException("Error binding socket");
	}
}

void ServerSocket::listen(){
	int err= ::listen(sock, SOMAXCONN);
	if(err < 0) {
		throwException("Error listening socket");
	}
}

Socket *ServerSocket::accept(){
	SOCKET newSock= ::accept(sock, NULL, NULL);
	if(newSock < 0) {
		throwException("Error accepting socket connection");
	}
	return new Socket(newSock);
}

}}//end namespace

